package com.migao.controller;


import cn.hutool.core.io.FileUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.migao.config.authority.AuthenticationProperties;
import com.migao.config.authority.LoginUtils;
import com.migao.config.constant.SystemConstant;
import com.migao.config.properties.SystemProperties;
import com.migao.config.response.ResponseBean;
import com.migao.config.response.ResponseBuilder;
import com.migao.entity.po.FileInfo;
import com.migao.entity.po.User;
import com.migao.entity.vo.res.FileRes;
import com.migao.service.FileService;
import com.migao.util.EntityUtils;
import com.migao.util.FileUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;


/**
 * @author : lcl
 * @date : 2020/10/14 9:24
 */
@Api(tags = "001.统一上传")
@RestController
@RequestMapping(value = "/file")
@Slf4j
public class FileInfoController {

    @Resource
    private FileService fileInfoService;
    @Resource
    private SystemProperties systemProperties;
    @Resource
    private AuthenticationProperties authenticationProperties;

    @ApiOperation(value = "01.文件统一上传")
    @PostMapping(value = "/upload")
    public ResponseBean<FileRes> upload(@RequestPart MultipartFile file) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();

        String originalFilename = file.getOriginalFilename();
        String prefix = FileUtils.getPrefix(originalFilename, SystemConstant.FILE_NAME_SPLIT_SYMBOL);
        String suffix = FileUtils.getSuffix(originalFilename, SystemConstant.FILE_NAME_SPLIT_SYMBOL);
        String fileNo = FileUtils.getFileNo();
        String uri = StringUtils.join(
                systemProperties.getFileTempDir(),
                SystemConstant.URI_CONCAT_SYMBOL,
                fileNo,
                SystemConstant.FILE_NAME_CONCAT_SYMBOL,
                suffix
        );
        String finalName = StringUtils.join(
                fileNo,
                SystemConstant.FILE_NAME_CONCAT_SYMBOL,
                suffix
        );
        String diskPath = StringUtils.join(
                systemProperties.getFileTempPath(),
                File.separator,
                finalName
        );
        String url = StringUtils.join(
                systemProperties.getDownloadPrefix(),
                uri
        );
        try {
            //写入文件
            FileUtils.storeFileByInputStream(
                    file.getInputStream(),
                    diskPath
            );
        } catch (IOException e) {
            e.printStackTrace();
            return ResponseBuilder.fail("文件上传失败");
        }

        FileInfo build = FileInfo
                .builder()
                .name(finalName)
                .originalName(prefix)
                .diskPath(diskPath)
                .size(file.getSize())
                .suffix(suffix)
                .uri(uri)
                .deleted(0)
                .createTime(LocalDateTime.now())
                .createUserId(user.getId())
                .build();
        fileInfoService.save(build);
        log.info("上传文件:{}", JSONObject.toJSONString(build));
        Map<String, Object> form = new HashMap<>();
        form.put("file", FileUtil.file(diskPath));
        systemProperties.getServerInfo().forEach(
                serverInfo -> {
                    String serverUpload = HttpRequest.post(
                            StringUtils.join(
                                    "http://",
                                    serverInfo.getServerAddress(),
                                    SystemConstant.URI_CONCAT_SYMBOL,
                                    "/file/serverUpload?id=",
                                    build.getId()
                            )
                    )
                            .header(authenticationProperties.getHeaderName(), "LoginUtils.getToken()")//头信息，多个头信息多次调用此方法即可
                            .form(form)//表单内容
                            .timeout(20000)//超时，毫秒
                            .execute().body();
                    log.debug("文件同步:{}", serverUpload);
                }
        );
        FileRes fileRes = EntityUtils.copyProperties(build, FileRes.class);
        fileRes.setUrl(
                StringUtils.join(
                        systemProperties.getDownloadPrefix(),
                        build.getUri()
                )
        );
        return ResponseBuilder.ok(fileRes);
    }


    @ApiOperation(value = "02.服务器文件同步", hidden = true)
    @PostMapping(value = "/serverUpload")
    public ResponseBean<FileRes> serverUpload(@RequestBody MultipartFile file, @RequestParam("id") Integer id) {
        FileInfo fileInfo = fileInfoService.getById(id);
        if (fileInfo == null) {
            return ResponseBuilder.fail();
        }
        //写入文件
        try {
            FileUtils.storeFileByInputStream(
                    file.getInputStream(),
                    fileInfo.getDiskPath()
            );
        } catch (IOException e) {
            e.printStackTrace();
        }
        FileRes fileRes = EntityUtils.copyProperties(fileInfo, FileRes.class);
        fileRes.setUrl(
                StringUtils.join(
                        systemProperties.getDownloadPrefix(),
                        fileInfo.getUri()
                )
        );
        return ResponseBuilder.ok(fileRes);
    }

}
