package com.migao.controller;


import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.validation.Insert;
import com.migao.config.validation.Update;
import com.migao.entity.vo.req.*;
import com.migao.entity.vo.res.FinancesQueryRes;
import com.migao.service.FinancesService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiOperationSupport;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import java.util.List;

@Api(tags = "006.财务管理")
@RestController
@RequestMapping("/finances")
public class FinancesController {

    @Resource
    private FinancesService financesService;

    @ApiOperationSupport(order = 2)
    @ApiOperation("新增付款申请")
    @PostMapping(value = "/insert")
    public ResponseBean<?> insert(
            @Validated(value = {Insert.class}) @RequestBody FinancesInsertReq financesInsertReq
    ) {
        return financesService.insert(financesInsertReq);
    }

    @ApiOperationSupport(order = 3)
    @ApiOperation("审批通过")
    @PostMapping(value = "/approve")
    public ResponseBean<?> approve(
            @RequestParam Integer id
    ) {
        return financesService.approve(id);
    }

    @ApiOperationSupport(order = 4)
    @ApiOperation("审批拒绝")
    @PostMapping(value = "/reject")
    public ResponseBean<?> reject(
            @RequestParam Integer id
    ) {
        return financesService.reject(id);
    }
    @ApiOperationSupport(order = 5)
    @ApiOperation("删")
    @PostMapping(value = "/deleteById")
    public ResponseBean<?> deleteById(
            @RequestParam Integer id
    ) {
        return financesService.deleteById(id);
    }
    @ApiOperationSupport(order = 6)
    @ApiOperation("根据id查询")
    @PostMapping(value = "/findById")
    public ResponseBean<?> findById(
            @RequestParam Integer id
    ) {
        return financesService.findById(id);
    }
    @ApiOperationSupport(order = 7)
    @ApiOperation("批量删除")
    @PostMapping(value = "/deleteBatch")
    public ResponseBean<?> deleteBatch(
            @RequestBody List<Integer> ids
    ) {
        return financesService.deleteBatch(ids);
    }

    @ApiOperationSupport(order = 8)
    @ApiOperation("修改")
    @PostMapping(value = "/update")
    public ResponseBean<?> update(
            @Validated(value = {Update.class}) @RequestBody FinancesUpdateReq financesUpdateReq
    ) {
        return financesService.update(financesUpdateReq);
    }

    @ApiOperationSupport(order = 1)
    @ApiOperation("分页查")
    @PostMapping(value = "/pageQuery")
    public ResponseBean<PageBean<FinancesQueryRes>> pageQuery(
            @RequestBody FinancesPageQueryReq financesPageQueryReq
    ) {
        return financesService.pageQuery(financesPageQueryReq);
    }
}
