package com.migao.entity.po;

import com.baomidou.mybatisplus.annotation.*;
import com.migao.config.constant.FinancesStatus;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@Entity
@Table(name = "t_finances")
@org.hibernate.annotations.Table(appliesTo = "t_finances", comment = "财务管理表")
@TableName("t_finances")
public class Finances implements Serializable {

    @Id
    @TableId(type = IdType.AUTO)
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", columnDefinition = "int COMMENT '主键'")
    private Integer id;

    @Column(name = "project_id", columnDefinition = "int COMMENT '项目编码'")
    private Integer projectId;

    @Column(name = "name", columnDefinition = "varchar(255) comment '预算项目名称'")
    private String name;

    @Column(name = "company_nme", columnDefinition = "varchar(255) comment '收款单位名称'")
    private String companyName ;

    @Column(name = "no", columnDefinition = "varchar(255) comment '收款单位供应商编码'")
    private String no;

    @Column(name = "bank_number", columnDefinition = "varchar(255) comment '收款单位开户联行号-银行存款'")
    private String bankNumber;

    @Column(name = "bank_address", columnDefinition = "varchar(255) comment '收款单位开户行-银行存款'")
    private String bankAddress;

    @Column(name = "bank_account", columnDefinition = "varchar(255) comment '收款单位银行账号-银行存款'")
    private String bankAccount;

    @Column(name = "bank_number_drafts", columnDefinition = "varchar(255) comment '收款单位开户联行号-电子汇票（大额支付必填）'")
    private String bankNumberDrafts;

    @Column(name = "bank_address_drafts", columnDefinition = "varchar(255) comment '收款单位开户行-电子汇票（大额支付必填）'")
    private String bankAddressDrafts;

    @Column(name = "bank_account_drafts", columnDefinition = "varchar(255) comment '收款单位银行账号-电子汇票（大额支付必填）'")
    private String bankAccountDrafts;

    @Column(name = "pay_money", columnDefinition = "decimal comment '付款金额（元）'")
    private BigDecimal payMoney;

    @Column(name = "contract_money", columnDefinition = "decimal comment '合同金额（元）'")
    private BigDecimal contractMoney;

    @Column(name = "remit_time", columnDefinition = "timestamp DEFAULT CURRENT_TIMESTAMP comment '预计付款时间'")
    private LocalDateTime remitTime;

    @Column(name = "another", columnDefinition = "varchar(255) comment '上级代付（大额支付）'")
    private String another;

    @Column(name = "username", columnDefinition = "varchar(255) comment '填报专责'")
    private String username;

    @Column(name = "purchase_no", columnDefinition = "varchar(255) comment 'erp采购订单号'")
    private String purchaseNo;

    @Column(name = "financial", columnDefinition = "varchar(255) comment '财务报表'")
    private String financial;

    @Column(name = "pay_no", columnDefinition = "varchar(255) comment '付款订单编号'")
    private String payNo;

    @Column(name = "organization", columnDefinition = "varchar(255) comment '采购组织(S201：工程服务，S202：维修服务）'")
    private String organization;

    @Column(name = "status", columnDefinition = "varchar(255) comment '状态'")
    private FinancesStatus status;

    @TableField(fill = FieldFill.INSERT_UPDATE)
    @Column(name = "deleted", columnDefinition = "tinyint(4) comment '逻辑删除 0未删除 1已删除'")
    private Integer deleted;

    @Column(name = "create_time", columnDefinition = "timestamp DEFAULT CURRENT_TIMESTAMP comment '创建时间'")
    private LocalDateTime createTime;

    @JoinColumn(name = "create_user_id", columnDefinition = "bigint(20) comment '新增人'")
    private Integer createUserId;
}
