package com.migao.entity.po;

import com.baomidou.mybatisplus.annotation.*;
import com.migao.config.constant.ProjectStatus;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.persistence.*;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;


@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@Entity
@Table(name = "t_project")
@org.hibernate.annotations.Table(appliesTo = "t_project", comment = "项目储备表")
@TableName("t_project")
public class Project implements Serializable {

    @Id
    @TableId(type = IdType.AUTO)
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", columnDefinition = "int COMMENT '主键'")
    private Integer id;

    @Column(name = "parent_id", columnDefinition = "int comment '父级id'")
    private Integer parentId;

    @Column(name = "no", columnDefinition = "varchar(255) comment '项目编号'")
    private String no;

    @Column(name = "content", columnDefinition = "varchar(255) comment '项目内容'")
    private String content;

    @Column(name = "name", columnDefinition = "varchar(255) comment '项目名称'")
    private String name;

    @Column(name = "unit", columnDefinition = "varchar(255) comment '填报单位'")
    private String unit;

    @Column(name = "project_classify", columnDefinition = "varchar(255) comment '项目分类'")
    private String projectClassify;

    @Column(name = "specialty_classify", columnDefinition = "varchar(255) comment '项目专业分类'")
    private String specialtyClassify;

    @Column(name = "two_classify", columnDefinition = "varchar(255) comment '二级分类'")
    private String twoClassify;

    @Column(name = "total_investment",columnDefinition = "decimal comment '总投资(万元)'")
    private BigDecimal totalInvestment;

    @Column(name = "total",columnDefinition = "decimal comment '合计(万元)'")
    private BigDecimal total;

    @Column(name = "capital",columnDefinition = "decimal comment '资本性(万元)'")
    private BigDecimal capital;

    @Column(name = "cost",columnDefinition = "decimal comment '成本性(万元)'")
    private BigDecimal cost;

    @Column(name = "level", columnDefinition = "varchar(255) comment '重要等级'")
    private String level;

    @Column(name = "progress", columnDefinition = "varchar(255) comment '可研情况'")
    private String progress;

    @Column(name = "reserve_status", columnDefinition = "varchar(255) comment '储备状态'")
    private String reserveStatus;

    @Column(name = "order_status", columnDefinition = "varchar(255) comment '下达状态'")
    private String orderStatus;

    @Column(name = "purchase_status", columnDefinition = "varchar(255) comment '采购审批状态'")
    private String purchaseStatus;


    @TableField(fill = FieldFill.INSERT_UPDATE)
    @Column(name = "deleted", columnDefinition = "tinyint(4) comment '逻辑删除 0未删除 1已删除'")
    private Integer deleted;

    @Column(name = "create_time", columnDefinition = "timestamp DEFAULT CURRENT_TIMESTAMP comment '储备年份'")
    private LocalDateTime createTime;

    @JoinColumn(name = "create_user_id", columnDefinition = "bigint(20) comment '新增人'")
    private Integer createUserId;
}
