package com.migao.entity.po;

import com.baomidou.mybatisplus.annotation.*;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import java.io.Serializable;
import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * @author dingsong
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@Entity
@Table(name = "t_supplier")
@org.hibernate.annotations.Table(appliesTo = "t_supplier", comment = "供应商")
@TableName("t_supplier")
public class Supplier implements Serializable {
    @Id
    @TableId(type = IdType.AUTO)
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id", columnDefinition = "int comment '主键'")
    private Integer id;

    @Column(name = "name", columnDefinition = "varchar(255) comment '公司名称'")
    private String name;

    @Column(name = "company_album", columnDefinition = "varchar(255) comment '公司资质'")
    private String companyAlbum;

    @Column(name = "company_size", columnDefinition = "varchar(255) comment '公司规模'")
    private String companySize;

    @Column(name = "company_type_id", columnDefinition = "bigint(20) comment '公司类别'")
    private Integer companyTypeId;

    @Column(name = "supplier_code", columnDefinition = "varchar(255) comment '供应商编码'")
    private String supplierCode;

    @Column(name = "license_number", columnDefinition = "bigint(20) comment '执照编号'")
    private Integer licenseNumber;

    @Column(name = "legal_person_name", columnDefinition = "varchar(255) comment '法人姓名'")
    private String legalPersonName;

    @Column(name = "phone_one", columnDefinition = "bigint(20) comment '法人手机号1'")
    private Integer phoneOne;

    @Column(name = "phone_two", columnDefinition = "bigint(20) comment '法人手机号2'")
    private Integer phoneTwo;

    @Column(name = "phone", columnDefinition = "bigint(20) comment '负责人手机号'")
    private Integer phone;

    @Column(name = "address", columnDefinition = "varchar(255) comment '注册地址'")
    private String address;

    @Column(name = "money", columnDefinition = "varchar(255) comment '注册资金'")
    private String money;

    @Column(name = "project_grades", columnDefinition = "varchar(255) comment '项目评分'")
    private String  projectGrades;

    @Column(name = "business_scope", columnDefinition = "text comment '公司营业范围'")
    private String  businessScope;

    @Column(name = "remark", columnDefinition = "text comment '备注'")
    private String  remark;

    @Column(name = "company_create_time", columnDefinition = "timestamp DEFAULT CURRENT_TIMESTAMP comment '公司成立时间'")
    private LocalDateTime companyCreateTime;

    @Column(name = "intranet_mail", columnDefinition = "varchar(255) comment '内网邮箱'")
    private String  intranetMail;

    @Column(name = "outside_mail", columnDefinition = "varchar(255) comment '外网邮箱'")
    private String  outsideMail;

    @Column(name = "opening_bank_number", columnDefinition = "varchar(255) comment '开户联行号'")
    private String  openingBankNumber;

    @Column(name = "opening_bank", columnDefinition = "varchar(255) comment '开户行'")
    private String  openingBank;

    @Column(name = "bank_account", columnDefinition = "varchar(255) comment '银行账号'")
    private String  bankAccount;

    @Column(name = "opening_bank_number_d", columnDefinition = "varchar(255) comment '开户联行号-电子'")
    private String  openingBankNumberD;

    @Column(name = "opening_bank_d", columnDefinition = "varchar(255) comment '开户行-电子'")
    private String  openingBankD;

    @Column(name = "bank_account_d", columnDefinition = "varchar(255) comment '银行账号-电子'")
    private String  bankAccountD;

    @TableField(fill = FieldFill.INSERT_UPDATE)
    @Column(name = "deleted", columnDefinition = "tinyint(4) comment '逻辑删除 0未删除 1已删除'")
    private Integer deleted;

    @Column(name = "create_time", columnDefinition = "timestamp DEFAULT CURRENT_TIMESTAMP comment '创建时间'")
    private LocalDateTime createTime;

    @JoinColumn(name = "create_user_id", columnDefinition = "bigint(20) comment '新增人'")
    private Integer createUserId;
}
