package com.migao.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.migao.config.constant.FinancesStatus;
import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.response.ResponseBuilder;
import com.migao.entity.po.Finances;
import com.migao.entity.vo.req.FinancesInsertReq;
import com.migao.entity.vo.req.FinancesPageQueryReq;
import com.migao.entity.vo.req.FinancesQueryReq;
import com.migao.entity.vo.req.FinancesUpdateReq;
import com.migao.entity.vo.res.FinancesQueryRes;
import com.migao.mapper.FinancesMapper;
import com.migao.mapper.FinancesRepository;
import com.migao.service.FinancesService;
import com.migao.util.EntityUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = {Exception.class})
public class FinancesServiceImpl implements FinancesService {

    @Resource
    private FinancesRepository financesRepository;

    @Resource
    private FinancesMapper financesMapper;


    /**
     * 增
     *
     * @param financesInsertReq financesInsertReq
     * @return ?
     */
    @Override
    public ResponseBean<?> insert(FinancesInsertReq financesInsertReq) {
        Finances finances = EntityUtils.copyProperties(financesInsertReq, Finances.class);
        finances.setStatus(FinancesStatus.TO_AUDIT);
        finances.setDeleted(0);
        financesRepository.save(finances);
        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<Finances> findById(Integer id) {
        Finances finances = financesMapper.selectById(id);
        return ResponseBuilder.ok(finances);
    }

    @Override
    public ResponseBean<?> approve(Integer id) {
        Finances finances = financesMapper.selectById(id);
        if (finances == null) {
            return ResponseBuilder.fail("记录不存在");
        }
        finances.setStatus(FinancesStatus.PASS);
        financesMapper.updateById(finances);
        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<?> reject(Integer id) {
        Finances finances = financesMapper.selectById(id);
        if (finances == null) {
            return ResponseBuilder.fail("记录不存在");
        }
        finances.setStatus(FinancesStatus.TURN);
        financesMapper.updateById(finances);
        return ResponseBuilder.ok();
    }

    /**
     * 分页查
     *
     * @param financesPageQueryReq financesPageQueryReq
     * @return ?
     */
    @Override
    public ResponseBean<PageBean<FinancesQueryRes>> pageQuery(FinancesPageQueryReq financesPageQueryReq) {
        Integer page = financesPageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = financesPageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;
        LambdaQueryWrapper<Finances> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotBlank(financesPageQueryReq.getName())) {
            queryWrapper.like(Finances::getName, "%" + financesPageQueryReq.getName() + "%");
        }
        if (StringUtils.isNotBlank(financesPageQueryReq.getStatus())) {
            queryWrapper.eq(Finances::getStatus, financesPageQueryReq.getStatus());
        }
        Page<Finances> all = financesMapper.selectPage(new Page<>(page, size), queryWrapper);
        return ResponseBuilder.ok(
                all,
                element -> {
                    FinancesQueryRes financesQueryRes = FinancesQueryRes
                            .builder()
                            .id(element.getId())
                            .another(element.getAnother())
                            .bankAccount(element.getBankAccount())
                            .bankAccountDrafts(element.getBankAccountDrafts())
                            .bankAddress(element.getBankAddress())
                            .bankAddressDrafts(element.getBankAddressDrafts())
                            .bankNumber(element.getBankNumber())
                            .bankNumberDrafts(element.getBankNumberDrafts())
                            .companyNme(element.getCompanyName())
                            .contractMoney(String.valueOf(element.getContractMoney()))
                            .createTime(element.getCreateTime())
                            .createUserId(element.getCreateUserId())
                            .deleted(element.getDeleted())
                            .financial(element.getFinancial())
                            .name(element.getName())
                            .no(element.getNo())
                            .organization(element.getOrganization())
                            .payMoney(String.valueOf(element.getPayMoney()))
                            .payNo(element.getPayNo())
                            .projectId(element.getProjectId())
                            .purchaseNo(element.getPurchaseNo())
                            .remitTime(element.getRemitTime())
                            .status(element.getStatus().getName())
                            .username(element.getUsername())
                            .build();
                    return financesQueryRes;
                }
        );
    }

    /**
     * 删
     *
     * @param id id
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteById(Integer id) {
        financesRepository.findById(id).ifPresent(
                i -> {
                    i.setDeleted(1);
                    financesRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 批量删
     *
     * @param ids ids
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteBatch(List<Integer> ids) {
        financesRepository.findAllById(ids).forEach(
                i -> {
                    i.setDeleted(1);
                    financesRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 改
     *
     * @param financesUpdateReq financesUpdateReq
     * @return ?
     */
    @Override
    public ResponseBean<?> update(FinancesUpdateReq financesUpdateReq) {
        financesRepository.save(EntityUtils.copyProperties(financesUpdateReq, Finances.class));
        return ResponseBuilder.ok();
    }
}
