package com.migao.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.migao.config.authority.LoginUtils;
import com.migao.config.response.*;
import com.migao.entity.po.DemandLibrary;
import com.migao.entity.po.Message;
import com.migao.entity.po.User;
import com.migao.entity.vo.req.MessageQueryReq;
import com.migao.entity.vo.res.MessageQueryRes;
import com.migao.mapper.DictionaryMapper;
import com.migao.mapper.MessageMapper;
import com.migao.mapper.UserMapper;
import com.migao.service.MessageService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * @author dingsong
 */
@Slf4j
@Service
@Transactional(rollbackFor = {Exception.class})
public class MessageServiceImpl extends ServiceImpl<MessageMapper, Message> implements MessageService {
    @Resource
    MessageMapper messageMapper;
    @Resource
    UserMapper userMapper;
    @Resource
    DictionaryMapper dictionaryMapper;

    @Override
    public ResponseBean<PageBeanOne<MessageQueryRes>> pageQuery(BasePageRequest basePageRequest) {
        Integer page = basePageRequest.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = basePageRequest.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Message> me = Wrappers.<Message>lambdaQuery();
        me.eq(Message::getDeleted,0);
        me.orderByDesc(Message::getCreateTime);

        Page<Message> all = messageMapper.selectPage(new Page<>(page, size),me);
        return ResponseBuilderOne.ok(
                all,
                element -> {
                    MessageQueryRes messageQueryRes = MessageQueryRes
                            .builder()
                            .id(element.getId())
                            .messageNumber(element.getMessageNumber())
                            .name(element.getName())
                            .content(element.getContent())
                            .sendUser(userMapper.selectById(element.getSendUserId()).getName())
                            .acceptUser(userMapper.selectById(element.getAcceptUserId()).getName())
                            .designProject(element.getDesignProject())
                            .messageType(dictionaryMapper.selectById(element.getMessageTypeId()).getValue())
                            .status(element.getStatus() == 0 ? "未读":"已读")
                            .build();
                       return messageQueryRes;
                   }
                );
    }

    @Override
    public ResponseBean<PageBeanOne<MessageQueryRes>> pageQueryById(BasePageRequest basePageRequest) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();

        Integer page = basePageRequest.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = basePageRequest.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Message> me = Wrappers.<Message>lambdaQuery();
        me.eq(Message::getDeleted,0);
        me.eq(Message::getAcceptUserId,user.getId());
        me.orderByDesc(Message::getCreateTime);

        Page<Message> all = messageMapper.selectPage(new Page<>(page, size),me);
        return ResponseBuilderOne.ok(
                all,
                element -> {
                    MessageQueryRes messageQueryRes = MessageQueryRes
                            .builder()
                            .id(element.getId())
                            .messageNumber(element.getMessageNumber())
                            .name(element.getName())
                            .content(element.getContent())
                            .sendUser(userMapper.selectById(element.getSendUserId()).getName())
                            .acceptUser(userMapper.selectById(element.getAcceptUserId()).getName())
                            .designProject(element.getDesignProject())
                            .messageType(dictionaryMapper.selectById(element.getMessageTypeId()).getValue())
                            .status(element.getStatus() == 0 ? "未读":"已读")
                            .build();
                    return messageQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<MessageQueryRes> pageQueryOne(String id) {
        Integer idInt = Integer.parseInt(id);
        Message message = messageMapper.selectById(idInt);

        MessageQueryRes messageQueryRes = MessageQueryRes
                .builder()
                .id(message.getId())
                .messageNumber(message.getMessageNumber())
                .name(message.getName())
                .content(message.getContent())
                .sendUser(userMapper.selectById(message.getSendUserId()).getName())
                .acceptUser(userMapper.selectById(message.getAcceptUserId()).getName())
                .designProject(message.getDesignProject())
                .messageType(dictionaryMapper.selectById(message.getMessageTypeId()).getValue())
                .status(message.getStatus() == 0 ? "未读":"已读")
                .build();


        return ResponseBuilder.ok(messageQueryRes);
    }

    @Override
    public ResponseBean<?> updateById(String id) {
        Integer idInt = Integer.parseInt(id);
        Message message = messageMapper.selectById(idInt);
        if(message.getStatus() == 0) {
            message.setStatus(1);
            messageMapper.updateById(message);
            return ResponseBuilder.ok(200,"更新完毕");
        }
        return ResponseBuilder.ok(200,"当前消息为已读状态，无需重复操作");
    }

    @Override
    public ResponseBean<PageBeanOne<MessageQueryRes>> pageQueryTwo(MessageQueryReq messageQueryReq) {
        Integer page = messageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = messageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Message> mee = Wrappers.<Message>lambdaQuery();
        if(StringUtils.isNotBlank(messageQueryReq.getKey())){
            //标题
            mee.like(Message::getName,messageQueryReq.getKey());
            //内容
            LambdaQueryWrapper<Message> or1 = mee.or();
            or1.like(Message::getContent,messageQueryReq.getKey());
            //设计项目
            LambdaQueryWrapper<Message> or2 = or1.or();
            or2.like(Message::getDesignProject,messageQueryReq.getKey());
        }
        mee.orderByDesc(Message::getCreateTime);
        Page<Message> all = messageMapper.selectPage(new Page<>(page, size),mee);

        return ResponseBuilderOne.ok(
                all,
                element -> {
                    MessageQueryRes messageQueryRes = MessageQueryRes
                            .builder()
                            .id(element.getId())
                            .messageNumber(element.getMessageNumber())
                            .name(element.getName())
                            .content(element.getContent())
                            .sendUser(userMapper.selectById(element.getSendUserId()).getName())
                            .acceptUser(userMapper.selectById(element.getAcceptUserId()).getName())
                            .designProject(element.getDesignProject())
                            .messageType(dictionaryMapper.selectById(element.getMessageTypeId()).getValue())
                            .status(element.getStatus() == 0 ? "未读":"已读")
                            .build();
                    return messageQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<PageBeanOne<MessageQueryRes>> pageQueryTwoById(MessageQueryReq messageQueryReq) {
        //获取当前登录用户的信息
        User user = LoginUtils.getUser();

        Integer page = messageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = messageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Message> mee = Wrappers.<Message>lambdaQuery();
        mee.eq(Message::getAcceptUserId,user.getId());
        if(StringUtils.isNotBlank(messageQueryReq.getKey())){
            //标题
            mee.like(Message::getName,messageQueryReq.getKey());
            //内容
            LambdaQueryWrapper<Message> or1 = mee.or();
            or1.like(Message::getContent,messageQueryReq.getKey());
            //设计项目
            LambdaQueryWrapper<Message> or2 = or1.or();
            or2.like(Message::getDesignProject,messageQueryReq.getKey());
        }
        mee.orderByDesc(Message::getCreateTime);
        Page<Message> all = messageMapper.selectPage(new Page<>(page, size),mee);

        return ResponseBuilderOne.ok(
                all,
                element -> {
                    MessageQueryRes messageQueryRes = MessageQueryRes
                            .builder()
                            .id(element.getId())
                            .messageNumber(element.getMessageNumber())
                            .name(element.getName())
                            .content(element.getContent())
                            .sendUser(userMapper.selectById(element.getSendUserId()).getName())
                            .acceptUser(userMapper.selectById(element.getAcceptUserId()).getName())
                            .designProject(element.getDesignProject())
                            .messageType(dictionaryMapper.selectById(element.getMessageTypeId()).getValue())
                            .status(element.getStatus() == 0 ? "未读":"已读")
                            .build();
                    return messageQueryRes;
                }
        );
    }


}
