package com.migao.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.response.ResponseBuilder;
import com.migao.entity.po.Milestone;
import com.migao.entity.vo.req.MilestoneInsertReq;
import com.migao.entity.vo.req.MilestonePageQueryReq;
import com.migao.entity.vo.req.MilestoneUpdateReq;
import com.migao.entity.vo.res.MilestoneQueryRes;
import com.migao.mapper.MilestoneMapper;
import com.migao.mapper.MilestoneRepository;
import com.migao.service.MilestoneService;
import com.migao.util.EntityUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

@Service
@Transactional(rollbackFor = {Exception.class})
public class MilestoneServiceImpl implements MilestoneService {


    @Resource
    private MilestoneMapper milestoneMapper;
    @Resource
    private MilestoneRepository milestoneRepository;


    /**
     * 增
     *
     * @param milestoneInsertReq milestoneInsertReq
     * @return ?
     */
    @Override
    public ResponseBean<?> insert(MilestoneInsertReq milestoneInsertReq) {
        Milestone milestone = new Milestone();
        BeanUtils.copyProperties(milestoneInsertReq, milestone);
        milestone.setDeleted(0);
        milestone.setSchedule("0");
        return ResponseBuilder.ok(milestoneMapper.insert(milestone));
    }

    /**
     * 分页查
     *
     * @param milestonePageQueryReq milestonePageQueryReq
     * @return ?
     */
    @Override
    public ResponseBean<PageBean<MilestoneQueryRes>> pageQuery(MilestonePageQueryReq milestonePageQueryReq) {
        //1。分页查询所有供公司
        Integer page = milestonePageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = milestonePageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Milestone> queryWrapper = new LambdaQueryWrapper<>();

        Page<Milestone> all = milestoneMapper.selectPage(new Page<>(page, size), queryWrapper);
        return ResponseBuilder.ok(
                all,
                element -> {
                    MilestoneQueryRes milestoneQueryRes = MilestoneQueryRes
                            .builder()
                            .id(element.getId())
                            .createTime(element.getCreateTime())
                            .createUserId(element.getCreateUserId())
                            .deleted(element.getDeleted())
                            .endTime(element.getEndTime())
                            .name(element.getName())
                            .projectId(element.getProjectId())
                            .schedule(element.getSchedule())
                            .status(element.getStatus())
                            .build();
                    return milestoneQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<?> importData(String fileId) {
        return null;
    }

    @Override
    public ResponseBean<?> export(MilestonePageQueryReq milestonePageQueryReq) {
        return null;
    }

    /**
     * 删
     *
     * @param id id
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteById(Integer id) {
        milestoneRepository.findById(id).ifPresent(
                i -> {
                    i.setDeleted(1);
                    milestoneRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 批量删
     *
     * @param ids ids
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteBatch(List<Integer> ids) {
        milestoneRepository.findAllById(ids).forEach(
                i -> {
                    i.setDeleted(1);
                    milestoneRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 改
     *
     * @param milestoneUpdateReq milestoneUpdateReq
     * @return ?
     */
    @Override
    public ResponseBean<?> update(MilestoneUpdateReq milestoneUpdateReq) {
        milestoneRepository.save(EntityUtils.copyProperties(milestoneUpdateReq, Milestone.class));
        return ResponseBuilder.ok();
    }

}
